package ch.zhaw.gartenverwaltung.backgroundtasks;

import ch.zhaw.gartenverwaltung.Settings;
import ch.zhaw.gartenverwaltung.backgroundtasks.email.EMailSender;
import ch.zhaw.gartenverwaltung.io.HardinessZoneNotSetException;
import ch.zhaw.gartenverwaltung.io.PlantList;
import ch.zhaw.gartenverwaltung.io.TaskList;
import ch.zhaw.gartenverwaltung.models.Garden;
import ch.zhaw.gartenverwaltung.types.Crop;
import ch.zhaw.gartenverwaltung.types.Task;

import javax.mail.MessagingException;
import java.io.IOException;
import java.time.LocalDate;

public class Notifier {
    private final TaskList taskList;
    private final Garden garden;
    private final PlantList plantList;
    private final EMailSender eMailSender = new EMailSender();

    public Notifier(TaskList taskList, Garden garden, PlantList plantList) {
        this.taskList = taskList;
        this.garden = garden;
        this.plantList = plantList;
    }

    private void sendNotification(Task task) {
        String plantName = "unkown plant";
        try {
            if(garden.getCrop(task.getCropId()).isPresent()){
                Crop crop = garden.getCrop(task.getCropId()).get();
                if(plantList.getPlantById(Settings.getInstance().getCurrentHardinessZone(), crop.getPlantId()).isPresent()) {
                    plantName = plantList.getPlantById(Settings.getInstance().getCurrentHardinessZone(), crop.getPlantId()).get().name();
                }
            }
        } catch (IOException | HardinessZoneNotSetException e) {
            e.printStackTrace();
            // TODO logger
        }
        String messageSubject = String.format(Settings.getInstance().getMailNotificationSubjectTemplate(), task.getName());
        String messageText = String.format(Settings.getInstance().getMailNotificationTextTemplate(), task.getName(), plantName, task.getNextExecution(), task.getDescription());
        try {
            eMailSender.sendMails(Settings.getInstance().getMailNotificationReceivers(), messageSubject, messageText);
        } catch (MessagingException e) {
            e.printStackTrace();
            // TODO Logger
        }
    }

    public void sendNotifications() throws IOException, MessagingException {
        System.out.println("sending Notifications");
        for (Task task : taskList.getTaskList(LocalDate.MIN, LocalDate.MAX)) {
            if (task.getNextNotification() != null && task.getNextNotification().isBefore(LocalDate.now().minusDays(1))) {
                sendNotification(task);
                task.setNextNotification(LocalDate.now().plusDays(1));
                taskList.saveTask(task);
            }
        }
    }
}
