package ch.zhaw.gartenverwaltung.io;

import ch.zhaw.gartenverwaltung.models.PlantNotFoundException;
import ch.zhaw.gartenverwaltung.types.*;

import java.io.IOException;
import java.time.LocalDate;
import java.util.List;

public class WeatherGradenTaskPlanner {
    private final TaskList taskList;
    private final PlantList plantList;
    private final CropList cropList;
    WeatherService weatherService;
    private final LocalDate dateSevereWeather = LocalDate.of(22,12,15);

    public WeatherGradenTaskPlanner(TaskList taskList, PlantList plantList, CropList cropList) {
        this.taskList = taskList;
        this.plantList = plantList;
        this.cropList = cropList;
        weatherService = new WeatherService();
    }

    public void refreshTasks() throws IOException, HardinessZoneNotSetException, PlantNotFoundException {
        getSevereWeatherEvents();
        getRainAmount();
    }

    private void getSevereWeatherEvents() throws IOException {
        SevereWeather actualWeather = weatherService.causeSevereWeather(0);
        if (SevereWeather.HAIL.equals(actualWeather)) {
            createPreHail();
        } else if (SevereWeather.FROST.equals(actualWeather)) {
            createPreFrost();
        } else if (SevereWeather.SNOW.equals(actualWeather)) {
            createPreSnow();
        }
    }

    private void getRainAmount() throws IOException, HardinessZoneNotSetException, PlantNotFoundException {
        int rainAmount = weatherService.causeRainAmount(3);
        adjustWateringTask(rainAmount);
    }

    public void createPreHail() throws IOException {

        Task preHailTask = new Task("Hail",
                                    "During a summer Thunderstorm it could hail heavily. THe Hail could damage the crops. To prevent damage cover the plants with a strong tarpaulin",
                                    dateSevereWeather.minusDays(1L));
        preHailTask.withId(Long.MAX_VALUE);
        taskList.saveTask(preHailTask);

    }

    public void createPreFrost() throws IOException {
        Task preFrostTask = new Task("Frost",
                                    "The temperatur falls below zero degrees, cover especially the root with wool",
                                    dateSevereWeather.minusDays(1L));
        preFrostTask.withId(Long.MAX_VALUE);
        taskList.saveTask(preFrostTask);
    }

    public void createPreSnow() throws IOException {
        Task preSnowTask = new Task("Snow",
                                    "The weather brings little snowfall. Cover your crops",
                                    dateSevereWeather.minusDays(1L));
        preSnowTask.withId(Long.MAX_VALUE);
        taskList.saveTask(preSnowTask);
    }

    public void adjustWateringTask(int rainAmount) throws HardinessZoneNotSetException, IOException, PlantNotFoundException {

        for (Crop crop : cropList.getCrops()) {
            Plant plant = plantList.getPlantById(HardinessZone.ZONE_8A,crop.getPlantId()).orElseThrow(PlantNotFoundException::new);

            // nur für aktuelle growthphase
            if(plant.wateringCycle().litersPerSqM() < rainAmount){
                adjustNextExecutionOfWateringTask(taskList.getTaskList(LocalDate.now(), LocalDate.now().plusDays(7)));
            }
        }

    }
// vom startdatum abhängig,
    private void adjustNextExecutionOfWateringTask(List<Task> cropTaskList){
        for(Task task : cropTaskList){
            if(task.getName().equals("watering task")){
                task.setNextExecution(task.getNextExecution().plusDays(task.getInterval().orElse(1)));
            }
        }
    }
}
